# -*- coding: utf-8 -*-

WS_CTE_RECEPCAO  = 0
WS_CTE_RET_RECEPCAO  = 1
WS_CTE_INUTILIZACAO  = 2
WS_CTE_CONSULTA  = 3
WS_CTE_STATUS_SERVICO = 4
WS_CTE_EVENTO  = 5
WS_CTE_CONSULTA_CADASTRO  = 6

CTE_AMBIENTE_PRODUCAO    = 1
CTE_AMBIENTE_HOMOLOGACAO = 2